package ru.ulstu.report.controller;

import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import ru.ulstu.aspirant.service.AspirantService;
import ru.ulstu.indicator.model.Indicator;
import ru.ulstu.indicator.service.IndicatorService;
import ru.ulstu.report.model.ReportListForm;
import ru.ulstu.report.model.ReportPeriod;
import ru.ulstu.report.model.dto.ReportDto;
import ru.ulstu.report.service.ReportPeriodService;
import ru.ulstu.report.service.ReportService;

import java.util.List;

@Controller
@RequestMapping("report")
public class ReportController {
    private final IndicatorService indicatorService;
    private final AspirantService aspirantService;
    private final ReportService reportService;
    private final ReportPeriodService reportPeriodService;

    public ReportController(IndicatorService indicatorService,
                            AspirantService aspirantService,
                            ReportService reportService,
                            ReportPeriodService reportPeriodService) {
        this.indicatorService = indicatorService;
        this.aspirantService = aspirantService;
        this.reportService = reportService;
        this.reportPeriodService = reportPeriodService;
    }

    @GetMapping("reportList")
    public String getReportPeriods(Model model) {
        List<ReportPeriod> periods = reportPeriodService.getReportPeriods();
        model.addAttribute("reportPeriods", periods);
        model.addAttribute("reportListForm", new ReportListForm(periods.isEmpty() ? null : periods.getFirst()));
        model.addAttribute("reports", reportService.getReports(periods.isEmpty() ? null : periods.getFirst()));
        model.addAttribute("canCreate", !periods.isEmpty() && reportService.canCreateReport(periods.getFirst()));
        return "report/reportList";
    }

    @PostMapping("reportList")
    public String getReportPeriods(ReportListForm reportListForm, Model model) {
        model.addAttribute("reportListForm", reportListForm);
        model.addAttribute("reports", reportService.getReports(reportListForm.getReportPeriod()));
        List<ReportPeriod> periods = reportPeriodService.getReportPeriods();
        model.addAttribute("reportPeriods", periods);
        model.addAttribute("canCreate", !periods.isEmpty() && reportService.canCreateReport(reportListForm.getReportPeriod()));
        return "report/reportList";
    }

    @GetMapping("new-report/{reportPeriodId}")
    public String createReport(@PathVariable("reportPeriodId") Integer reportPeriodId,
                               Model model) {
        List<Indicator> indicators = aspirantService.getCurrentAspirantIndicators();
        model.addAttribute("report", new ReportDto(reportPeriodId, indicators));
        return "report/editReport";
    }

    @GetMapping("edit-report/{reportId}")
    public String editReportWithoutReportPeriod(@PathVariable("reportId") Integer reportId, Model model) {
        List<Indicator> indicators = aspirantService.getCurrentAspirantIndicators();
        model.addAttribute("report", new ReportDto(reportService.getReportById(reportId), indicators));
        return "report/editReport";
    }

    @PostMapping(value = "save-report", params = "save")
    public String saveReport(@ModelAttribute("report") ReportDto reportDto, Model model) {
        reportService.saveReport(reportDto);
        return "redirect:/report/reportList";
    }

    @PostMapping(value = "save-report", params = "delete")
    public String deleteReport(@ModelAttribute("report") ReportDto reportDto) {
        reportService.deleteReport(reportDto.getId());
        return "redirect:/report/reportList";
    }
}
